#! /bin/sh
#
#  mkcert.sh
#
#  $Id: mkcert.sh,v 1.2 2004/08/26 16:00:05 source Exp $
#
#  This is a short script to quickly generate a self-signed X.509 key for
#  use in the OpenLink secure servers
#
#  (C)Copyright 2004 OpenLink Software.
#  All Rights Reserved.
#
#  The copyright above and this notice must be preserved in all
#  copies of this source code.  The copyright above does not
#  evidence any actual or intended publication of this source code.
#
#  This is unpublished proprietary trade secret of OpenLink Software.
#  This source code may not be copied, disclosed, distributed, demonstrated
#  or licensed except as authorized by OpenLink Software.
#

OPENSSL=openssl

randfile=randfile.$$
conffile=conffile.$$

# Test for -n or \c in echo command
if (echo "testing\c") | grep c >/dev/null; then
  # Stardent Vistra SVR4 grep lacks -e, says ghazi@caip.rutgers.edu.
  if (echo -n testing) | sed s/-n/xn/ | grep xn >/dev/null; then
    echo_n= 
    echo_c='
' 
  else
    echo_n=-n 
    echo_c= 
  fi
else
  echo_n= 
  echo_c='\c'
fi

usage() {
	echo "Usage: $0 keyfile"
	exit 1
}

# Make sure OpenSSL is installed
echo $echo_n "Checking OpenSSL: $echo_c"
$OPENSSL version 2>/dev/null || {
	echo OpenSSL was not found on this system.
	exit 1
}

#  parse command line into arguments
opts=`getopt 'c:' $*`
[ $? != 0 ] && usage
set -- $opts
while [ $1 != -- ]
do
	case $1 in
	-c)     # cert
		echo cert $2
		shift
		echo
		;;
	*)
		usage
		;;
	esac
	shift   # next flag
done
shift   # skip double dash

# Check options
[ $# -ne 1 ] && usage

keyfile="$1.key"
pubfile="$1.cer"
p12file="$1.pfx"

# Make sure we don't overwrite
[ -f $keyfile ] && {
	echo "$keyfile already exists."
#	exit 1
}
#[ -f $pubfile ] && {
#	echo "$pubfile already exists."
#	exit 1
#}

# Construct config file for OpenSSL
cat >$conffile <<EOF
RANDFILE	= $randfile
encrypt_key	= yes
distinguished_name = req_dn
x509_extensions	= v3ext
prompt		= yes
default_md	= md5

[ req_dn ]
CN		= Host name (externally visible)
CN_default	= host.example.com
CN_max		= 64

O		= Organization (eg. company)
O_default	= OpenLink Software

OU		= Organizational Unit
OU_default	= Research & Development

ST		= State or Province Name (full name)
ST_default	=

L		= Locality Name (eg. city)
L_default	=

C		= Country Name (2 letter code)
C_default	= US
C_min		= 2
C_max		= 2

[ v3ext ]
nsCertType	= server
nsComment	= OpenLink Secure Server
EOF

# Create empty cert with prudent permissions
cp /dev/null $keyfile
chmod 600 $keyfile
#chown root $keyfile

# Cleanup handler
cleanup() {
	rm -f $keyfile $randfile $conffile
	echo '*** FAILED TO GENERATE A CERTIFICATE ***'
	exit 1
}

# Create X509 certificate
$OPENSSL req -new -x509 -nodes -days 3650 -set_serial 1 \
	-config $conffile -out $keyfile -keyout $keyfile -text || cleanup

# Add DH parameters
$OPENSSL gendh -rand $randfile 512 >>$keyfile || cleanup

# Create binary certificate (public part only) that can be used for client
# trust (Eg. JDBC client)
$OPENSSL x509 -inform PEM -in $keyfile -outform DER -out $pubfile

# Create PKCS12 for other purposes
#$OPENSSL pkcs12 -export -in $keyfile -out $p12file

# Show  fingerprint
$OPENSSL x509 -subject -dates -fingerprint -noout -in $keyfile  || cleanup

rm -f $randfile $conffile

echo
echo "Created private key parameters in $keyfile"
echo "Created public key parameters in $pubfile"

exit 0
