#
#  forms.tcl
#
#  $Id: forms.tcl,v 1.3 2003/03/10 12:53:56 source Exp $
#
#  Very Basic Form management
#
#  (C)Copyright 1997 OpenLink Software.
#  All Rights Reserved.
#
#  The copyright above and this notice must be preserved in all
#  copies of this source code.  The copyright above does not
#  evidence any actual or intended publication of this source code.
#
#  This is unpublished proprietary trade secret of OpenLink Software.
#  This source code may not be copied, disclosed, distributed, demonstrated
#  or licensed except as authorized by OpenLink Software.
#

itcl_class RecordSet {
    public labels {}
    public FinalRow {}
    public ncols 0
    public nrows 0
    public data {}
    public keyField 0

    constructor {config} {}
    method configure {config} {}

    method resetData {} {
        set data {}
	set nrows 0
    }
    method addRow {args} {
	set row [eval list $args]
        lappend data $row
	set nc [llength $row]
	if {$nc > $ncols} { set ncols $nc }
	while {$nc > [llength $FinalRow]} {
	    lappend FinalRow {}
	}
	incr nrows
    }
    method numRows {} {
	return $nrows
    }
    method numCols {} {
	return $ncols
    }
    method getRow {n} {
	return [lindex $data $n]
    }
    method getLabel {n} {
	return [lindex $labels $n]
    }
    method fetch {key} {
	for {set row 0} {$row < $nrows} {incr row} {
	    set r [getRow $row]
	    if {$key == [keyOfRow $r]} { return $r }
	}
	return {}
    }
    method keyOfRow {row} {
	return [lindex $row $keyField]
    }
    method keys {} {
	set ret {}
	for {set row 0} {$row < $nrows} {incr row} {
	    lappend ret [keyOfRow [lindex $data $row]]
	}
	return $ret
    }
    method getFinalRow {} {
	return $FinalRow
    }
}

itcl_class Form {
    public data
    public values
    public noUpdate {}
    protected _txclass

    constructor {txclass} {
	set data $this.data
	set values ${this}_values
	set _txclass $txclass
	global $values
	RecordSet $data
	$data configure -labels { "Name" "Label" "Type" "Size" "Value" }
    }
    method configure {config} {}
    method addField {name label type size value} {
	$data addRow $name $label $type $size $value
	global $name
	if { $type == "select" || $type == "radio" } {
	    set $name [lindex $value 0]
	} else {
	    set $name $value
	}
    }
    method createHTML {base button} {
	html "<form method=post action=$base>"
	html "<table border=1 nowrap>"
	set colno -1
	foreach var [$data keys] {
	    global $var
	    eval set val $$var
	    set row [$data fetch $var]
	    set label [lindex $row 1]
	    set type [lindex $row 2]
	    set size [lindex $row 3]
	    html "<tr><td>$label:<td>"
	    incr colno
	    if { [lsearch $noUpdate $colno] >= 0 } {
		html $val
	    } elseif { $type == "checkbox" } {
		html "<input name=$var type=checkbox"
		if { $val == "Y" || $val == "1" } { html " checked" }
		html ">"
	    } elseif { $type == "select" } {
		html "<select name=$var"
		if { $size > 0 } { html " size=$size" }
		html ">"
		set values [lindex $row 4]
		set idx 0
		foreach i $values {
		    html "<option value=$idx"
		    if { $i == $val } { html " selected" }
		    html ">$i"
		    incr idx
		}
		html "</select>"
	    } elseif { $type == "radio" } {
		set values [lindex $row 4]
		set idx 0
		foreach i $values {
		    html "<input name=$var type=radio value=$idx"
		    if { $i == $val } { html " checked" }
		    html ">$i"
		    incr idx
		}
	    } else {
		html "<input name=$var type=$type size=$size value=\""
		html -escape $val
		html "\">"
	    }
	    html <br>
	}
	html "</table>"
	html "<br>"
	html "<input type=submit value=\" $button \">"
	html "<input type=reset value=\" Reset \">"
	TXbegin $_txclass
	html "</form>"
    }
    method handlePost {} {
	foreach var [$data keys] {
	    global $var
	    eval set val $$var
	    set row [$data fetch $var]
	    set label [lindex $row 1]
	    set type [lindex $row 2]
	    set size [lindex $row 3]
	    if { $type == "checkbox" } {
		set tst [string toupper [var $var $val]]
		if {"$tst" == "ON" || [string match $tst "Y*"]} {
		    set tst "Y"
		} else {
		    set tst "N"
		}
		set $var $tst
	    } elseif { $type == "select" || $type == "radio" } {
		set values [lindex $row 4]
		set newval [var $var $val]
		if {$newval >= 0 && $newval < [llength $values]} {
		    set $var [lindex $values $newval]
		}
	    } else {
		set $var [var $var $val]
	    }
	}
    }
}

proc selectList {data base fieldList actList stList} {
    html "<table border=1 nowrap width=95%>\n<tr>"
    foreach i $fieldList {
	set label [$data getLabel $i]
	html "<th align=left>$label</th>"
    }
    set i [llength $actList]
    if {$i < [llength $stList]} {
	set i [llength $stList]
    }
    html "<th colspan=$i align=center>action</th></tr>\n"
    set ncols [$data numCols]
    for { set j 0 } { $j < [$data numRows] } { incr j } {
	set row [$data getRow $j]
	html "<tr>"
	foreach i $fieldList {
	    html "<td>[lindex $row $i]&nbsp;</td>"
	}
	set key [escape [$data keyOfRow $row]]
	foreach i $actList {
	    html "<td align=center>"
	    html "<a href=\"${base}$key&cmd=[escape $i]\">$i</a></td>"
	}
	html "</TR>\n"
    }
    if {[llength $stList] != 0} {
	html "<tr>"
	foreach i [$data getFinalRow] {
	    html "<td>$i</td>"
	}
	foreach i $stList {
	    html "<td align=center><a href=\"${base}&cmd=[escape $i]\">$i</a></td>"
	}
	html "</TR>\n"
    }
    html "</TABLE><br>\n"
}
