#
#  odbcdsn.tcl
#
#  $Id: odbcdsn.tcl,v 1.31 2003/03/18 13:40:08 source Exp $
#
#  ODBC routines
#
#  (C)Copyright 1998 OpenLink Software.
#  All Rights Reserved.
#
#  The copyright above and this notice must be preserved in all
#  copies of this source code.  The copyright above does not
#  evidence any actual or intended publication of this source code.
#
#  This is unpublished proprietary trade secret of OpenLink Software.
#  This source code may not be copied, disclosed, distributed, demonstrated
#  or licensed except as authorized by OpenLink Software.
#

set dsnList ""
set dsnNameDescList ""

set ODBC_INI	    HKEY_LOCAL_MACHINE\\Software\\ODBC\\ODBC.INI
set ODBCINST_INI    HKEY_LOCAL_MACHINE\\Software\\ODBC\\ODBCINST.INI
set OPENLINK_INI    HKEY_LOCAL_MACHINE\\Software\\ODBC\\OPENLINK.INI


# List of Installed drivers (filled in at init)
set InstalledDrivers    {}

# undefined
set undef "<UNDEF>"

#
# Initialize dsn routines (internal use)
# Return 1 on error
#
proc DSNInit {} {
    global dsninit env
    if {[info exist dsninit]} {
	return 0
    }
    if {[os type] != "win32"} {
	set ini ".odbc.ini"
	if {[info exist env(HOME)]} {
	    set ini "$env(HOME)/.odbc.ini"
	}
	if {[info exist env(ODBCINI)]} {
	    set ini $env(ODBCINI)
	}

	if {[catch {
	    profile odbcini $ini r
	    }]} {
	    dsnfail "No OpenLink ODBC.INI file found."
	    return 1
	}
	set ini ".odbcinst.ini"
	if {[info exist env(HOME)]} {
	    set ini "$env(HOME)/.odbcinst.ini"
	}
	if {[info exist env(ODBCINSTINI)]} {
	    set ini $env(ODBCINSTINI)
	}

	if {[catch {
	    profile odbcinstini $ini r
	    }]} {
	    dsnfail "No OpenLink ODBCINST.INI file found."
	    return 1
	}
    }
    set dsninit 1
    return 0
}

#
# Initialize unix lite OPENLINK.INI routines
# Return 1 on error
#
proc DSNInit2 {} {
	global dsninit2 env
	if {[info exist dsninit2]} {
	    return 0
	}

	if {[info exist env(OPENLINKINI)]} {
	    set ini $env(OPENLINKINI)
	} else {
	    set ini [fnsearch "openlink.ini" $env(PATH) "../openlink.ini"]
	}

	if {[catch {
	    profile openlinkini $ini r
	    }]} {
	    dsnfail "No OpenLink OPENLINK.INI file found."
	    return 1
	}
	set dsninit2 1
	return 0
}

#
# Return all ODBC Data Sources ie. List of DSNs
#
proc GetDSNList {} {
    global ODBC_INI dsnList
    if {$dsnList == ""} {
	if {[DSNInit]} { return {} }
	if {[os type] == "win32"} {
	    catch { set dsnList [registry values "$ODBC_INI\\ODBC Data Sources"] }
	} else {
	    set dsnList [odbcini keys "ODBC Data Sources"]
	}
    }

    return [lsort2 $dsnList]
}

#
# Return all ODBC Data Sources as pairs of {name description}
#
proc GetDSNNameDescList {} {
    global ODBC_INI dsnNameDescList
    if {$dsnNameDescList == ""} {
	foreach dsn [GetDSNList] {
	    lappend dsnNameDescList [list $dsn [GetDSNValue $dsn "Description"]]
	}
    }

    return $dsnNameDescList
}

#
# Return the driver name for a datasource
#
proc GetDSNDriverName {dsn} {
    return [GetDSNValue "ODBC Data Sources" $dsn]
}

#
# Get a value from ODBC.INI
#
proc GetDSNValue {sect var} {
    global ODBC_INI undef

    set value ""
    if {[DSNInit]} { return $value }
    if {[os type] == "win32"} {
	if {$var == "UserName"} { set var "UID" }
	catch { set value [registry get "$ODBC_INI\\$sect" $var] }
    } else {
	set value [odbcini read $sect $var ""]
    }
    return $value
}

proc GetDSNValue2 {sect var def} {
    global ODBC_INI undef

    set value $def
    if {[DSNInit]} { return $value }
    if {[os type] == "win32"} {
	if {$var == "UserName"} { set var "UID" }
	catch { set value [registry get "$ODBC_INI\\$sect" $var] }
    } else {
	set value [odbcini read $sect $var $def]
    }
    return $value
}

#
# Get a value from ODBCINST.INI
#
proc GetODBCINSTValue {sect var} {
    global ODBCINST_INI undef

    set value $undef
    if {[DSNInit]} { return $value }
    if {[os type] == "win32"} {
	catch { set value [registry get "$ODBCINST_INI\\$sect" $var] }
    } else {
	set value [odbcinstini read $sect $var $undef]
    }
    return $value
}

#
# Get a value from OPENLINK registry or ODBC.INI
#
proc GetOPENLINKValue {sect var defval} {
    global OPENLINK_INI

    set value $defval
    if {[DSNInit]} { return $value }
    if {[os type] == "win32"} {
	catch { set value [registry get "$OPENLINK_INI\\$sect" $var] }
    } else {
	set value [odbcini read $sect $var $defval]
    }
    return $value
}

#
# Set a value in OPENLINK registry or ODBC.INI
#
proc SetOPENLINKValue {sect var val} {
    global OPENLINK_INI undef
    if {[os type] != "win32"} {
	if {[DSNInit]} { return }
	if {$val == $undef} {
	    odbcini write $sect $var
	} else {
	    odbcini write $sect $var $val
	}
	odbcini commit
    } else {
	if {$val == $undef} {
	    catch { registry delete "$OPENLINK_INI\\$sect" $var }
	} else {
	    catch { registry set "$OPENLINK_INI\\$sect" $var $val }
	}
    }
}

#
# Get a list from OPENLINK registry or ODBC.INI
#
proc GetOPENLINKList {sect} {
    global OPENLINK_INI

    if {[DSNInit]} { return {} }
    set plist {}
    if {[os type] != "win32"} {
	set plist [odbcini keys $sect]
    } else {
	catch { set plist [registry values "$OPENLINK_INI\\$sect"] }
    }
    return $plist
}

#
# Get a value from real OPENLINK.INI file
#
proc GetOPENLINKINIValue {sect var defval} {
    if {[DSNInit2]} { return $defval }
    return [openlinkini read $sect $var $defval]
}

#
# Set a value in real OPENLINK.INI file
#
proc SetOPENLINKINIValue {sect var val} {
    global undef
    if {[DSNInit2]} { return }
    if {$val == $undef} {
	openlinkini write $sect $var
    } else {
	openlinkini write $sect $var $val
    }
    openlinkini commit
}

#
# Get a list from real OPENLINK.INI file
#
proc GetOPENLINKINIList {sect} {
    global undef
    if {[DSNInit2]} { return $undef }
    set plist {}
    set plist [openlinkini keys $sect]
    return $plist
}

#
# Get a List of Parameters for the DSN
#
proc GetDSNParameterList {dsn} {
    global ODBC_INI undef

    if {[DSNInit]} { return {} }
    set plist $undef
    if {[os type] != "win32"} {
	set plist [odbcini keys $dsn]
    } else {
	catch { set plist [registry values "$ODBC_INI\\$dsn"] }
    }
    return $plist
}

# Return TRUE if driver is installed (Openlink can handle it)
proc DSNisOpenlink {dsn} {
    set state [GetODBCINSTValue "ODBC Drivers" [GetDSNDriverName $dsn]]
    return [expr {$state == "Installed"}]
}

#
# Check if ODBC is installed.
# Returns 1 if ODBC Installed
#
proc CheckODBCInstalled {} {
    global InstalledDrivers undef

    #if {[database isudbc]} {
	#dsnfail "Please use UDBC DSNs."
	#return 0
    #}
    set InstalledDrivers [DSNDriverList]
    if {$InstalledDrivers == $undef} {
	# error already reported
	return 0
    }
    if {$InstalledDrivers == {}} {
	dsnfail "No ODBC drivers installed on the server."
	return 0
    }
    return 1;
}

proc dsnfail {reason} {
    html <html><body>
    html <center><h2>Error</h2>
    html <h3>$reason</h3>
    html {<form><input type="button" name="exit" value="Exit" onclick="self.close()"></form></center></body></html>}
}

#
# Return ODBC or UDBC selected by current TCL-DB support
#
proc GetDBC {} {
    if {[database isudbc]} {
	return "UDBC"
    } else {
	return "ODBC"
    }
}

#
# Return list of server types
#
proc GetSVTypeList {} {
    global svTypes svTypesFixed initemplateini env
    set svTypesFixed 0
    set svTypes ""

    catch {
      # get initial list from templates.ini
      if {![info exist initemplateini]} {
	  profile templateini [fnsearch "template.ini" $env(TCLINCLUDE)]
	  set initemplateini 1
      }
      set svTypes [templateini keys "Server Types"]

      if {[os type] == "win32"} {
	# Additional list from registry
	foreach item [registry values "HKEY_LOCAL_MACHINE\\Software\\ODBC\\OPENLINK.INI\\Server Types"] {
	  if {[lsearch $svTypes $item] < 0} {
	    lappend svTypes $item
	  }
	}
      }
    }
    return [lsort2 $svTypes]
}


#
# Return list of server types as pairs of {type description}
#
proc GetSVTypeDescList {} {
    set svTypeDescList ""

    foreach type [GetSVTypeList] {
	set desc ""
	catch {set desc [GetSVType $type ""] }
	lappend svTypeDescList [list $type $desc]
    }

    return $svTypeDescList
}

#
# Return Description for a Server type
#
proc GetSVType {st def} {
    global initemplateini env undef
    set desc $def

    catch {
      if {[os type] != "win32"} {
	# get desc from templates.ini
        if {![info exist initemplateini]} {
	    profile templateini [fnsearch "template.ini" $env(TCLINCLUDE)]
	    set initemplateini 1
	}
	set desc [templateini read "Server Types" $st $def]
      } else {
	# get desc from registry
	set desc [registry get "HKEY_LOCAL_MACHINE\\Software\\ODBC\\OPENLINK.INI\\Server Types" $st $def]
      }
    }
    return $desc
}

#
# Write a server type and desc (or delete it)
#
proc SetSvType {st desc} {
    global initemplateini undef env
    if {[os type] != "win32"} {
	# write to templates.ini
        if {![info exist initemplateini]} {
	    profile templateini [fnsearch "template.ini" $env(TCLINCLUDE)]
	    set initemplateini 1
	}
	if {$desc == $undef} {
	    templateini write "Server Types" $st
	} else {
	    templateini write "Server Types" $st $desc
	}
	templateini commit
    } else {
	# write to registry
	if {$desc == $undef} {
	    catch { registry delete "HKEY_LOCAL_MACHINE\\Software\\ODBC\\OPENLINK.INI\\Server Types" $st }
	} else {
	    catch { registry set "HKEY_LOCAL_MACHINE\\Software\\ODBC\\OPENLINK.INI\\Server Types" $st $desc }
	}
    }
}

#
# Remove the DSN
#
proc DSNDelete {dsn} {
global ODBC_INI
    if {[os type] != "win32"} {
	odbcini write $dsn
	odbcini write "ODBC Data Sources" $dsn
	odbcini commit
    } else {
	catch {
	    registry delete "$ODBC_INI\\$dsn"
	    registry delete "$ODBC_INI\\ODBC Data Sources" $dsn
	}
    }
}

#
# Save the DSN.
# Cfg is array of parameters
#
proc DSNSave {dsn} {
    global dsnCfg undef ODBC_INI
    set DriverName $dsnCfg(DriverName)
    set dsnCfg(DriverName) $undef
    if {[os type] != "win32"} {
	foreach i [array names dsnCfg] {
	    if {$dsnCfg($i) == $undef} {
		odbcini write $dsn $i
	    } else {
		odbcini write $dsn $i $dsnCfg($i)
	    }
	}
	odbcini write "ODBC Data Sources" $dsn $DriverName
	odbcini commit
    } else {
	foreach i [array names dsnCfg] {
	    if {$i == "UserName"} {
		set j "UID"
	    } else {
		set j $i
	    }
	    if {$dsnCfg($i) == $undef} {
		catch { registry delete "$ODBC_INI\\$dsn" $j }
	    } else {
		catch { registry set "$ODBC_INI\\$dsn" $j $dsnCfg($i) }
	    }
	}
	catch { registry set "$ODBC_INI\\ODBC Data Sources" $dsn $DriverName}
    }
}

#
# Save the DSN version 2.
# Cfg is array of parameters
#
proc DSNSave2 {dsn} {
    global dsnCfg undef
    set DriverName $dsnCfg(DriverName)
    set dsnCfg(DriverName) $undef
    foreach i [array names dsnCfg] {
	SetDSNValue $dsn $i $dsnCfg($i)
    }
    SetDSNValue "ODBC Data Sources" $dsn $DriverName
}

proc SetDSNValue {sect var val} {
    global ODBC_INI undef
    if {[os type] != "win32"} {
	if {$val == $undef} {
	    odbcini write $sect $var
	} else {
	    odbcini write $sect $var $val
	}
	odbcini commit
    } else {
	if {$val == $undef} {
	    catch { registry delete "$ODBC_INI\\$sect" $var }
	} else {
	    catch { registry set "$ODBC_INI\\$sect" $var $val }
	}
    }
}

#
# Return List of installed drivers or undef on error
#
proc DSNDriverList {} {
    global ODBCINST_INI undef
    if {[DSNInit]} { return $undef }
    set driverlist {}
    if {[os type] == "win32"} {
	catch { set driverlist [registry values "$ODBCINST_INI\\ODBC Drivers"] }
    } else {
	set driverlist [odbcinstini keys "ODBC Drivers"]
    }

    return [lsort2 $driverlist]
}

#
# Return Wizard style for a driver
#
proc DSNDriverWizard {drivername} {
    return [ReadDriversIni "[os type] drivers" $drivername]
}

#
# Return Default value for parameter in Wizard set
#
proc GetWizardDefault {wizard param} {
    return [ReadDriversIni "$wizard Defaults" $param]
}

#
# Return list of parameters in Wizard set
#
proc GetWizardParams {wizard} {
    return [ListDriversIni "$wizard Defaults"]
}

#
# Return Value from drivers.ini file
#
proc ReadDriversIni {sect key} {
global undef env
    set value $undef
    if {[catch {
	profile driversini [fnsearch "drivers.ini" $env(TCLINCLUDE)] r
	}]} {
	dsnfail "Cant read drivers.ini"
	return ""
    }
    catch { set value [driversini read $sect $key] }
    return $value
}

#
# Return list of keys from a drivers.ini section
#
proc ListDriversIni {sect} {
global undef env
    set value {}
    if {[catch {
	profile driversini [fnsearch "drivers.ini" $env(TCLINCLUDE)] r
	set value [driversini keys $sect]
	}]} {
	dsnfail "Cant read drivers.ini"
    }
    return $value
}


#
#  Read list of similar environments for lite
#
proc ULiteEnvList {} {
  global undef dsnCfg

  if {[DSNInit2]} { return }

  set newlist {}

  #
  #  Read all key,value pairs from Lite Environments section
  #
  catch { openlinkini readall envlist "Lite Environments" }

  #
  #  Add base type, servertype and current environment name
  #
  set type $dsnCfg(BaseType)
  set envlist($type) $type
  set envlist($dsnCfg(ServerType)) $type
  set envlist($dsnCfg(EnvName)) $type

  #
  #  Create unique list of all environments with the same base type
  #
  foreach env [array names envlist] {
    if {$type == $envlist($env) && [lsearch $newlist $env] < 0} {
      lappend newlist $env
    }
  }

  #
  #  Return sorted list
  #
  return [lsort $newlist]
}


#
#  Link environment name to base type
#
proc ULiteEnvSave {env svt} {
  if {$env != $svt} {
    SetOPENLINKINIValue "Lite Environments" "$env" "$svt"
  }
}


#
# Return DLL(driver) name for a specified driver
#
proc GetDSNDriver {drivername} {
    return [GetODBCINSTValue $drivername "Driver"]
}


#
#  Once we have iodbcinst functionality we can use the real check
#  but for now we need to use
#
proc CheckValidDSN {dsn} {
  if {[regexp {^.*([{}(),;?\*=!@\\]|\[|\])} $dsn]} {
     return 0;
  }
  return 1;
}


#
#  Calculate label for Options field based on ServerType
#
proc domain_label {type desc} {
  global dsnCfg undef

  #
  #  Check for custom label
  #
  if {$desc == ""} { set desc [GetSVType $type ""] }

  #
  #  Calculate label
  #
  if {$desc == ""} {
    switch -glob -- [string tolower $type] {
      "db2"		{ set desc "Server/Adapter" }
      "informix*"	{ set desc "Informix Server" }
      "ingres*"		{ set desc "Ingres Net (vnode)" }
      "mysql"		{ set desc "Connection Options" }
      "odbc"		{ set desc "" }
      "openingres*"	{ set desc "Ingres Net (vnode)" }
      "oracle 8*"	{ set desc "NET 8 Service name" }
      "oracle 9*"	{ set desc "NET 9 Service name" }
      "oracle *"	{ set desc "SQL*NET Alias" }
      "postgresql"	{ set desc "DB Connection Options" }
      "progress*"	{ set desc "DB Connection Options" }
      "sqlserver*"	{ set desc "Connection Options" }
      "sybase*"		{ set desc "Connection Options" }
      "virtuoso"	{ set desc "" }
      default		{ set desc "Options" }
    }
  } 

  return $desc
}
