#
#  rulebook.tcl
#
#  $Id: rulebook.tcl,v 1.2 2000/10/25 20:04:52 source Exp $
#
#  Code to read and cache the rulebook
#
#  (C)Copyright 1997 OpenLink Software.
#  All Rights Reserved.
#
#  The copyright above and this notice must be preserved in all
#  copies of this source code.  The copyright above does not
#  evidence any actual or intended publication of this source code.
#
#  This is unpublished proprietary trade secret of OpenLink Software.
#  This source code may not be copied, disclosed, distributed, demonstrated
#  or licensed except as authorized by OpenLink Software.
#

set allSections ""
set agentSections ""
set mappingRules ""
set allDatabases {}

proc initRulebook {} {
    global env rulebookFile

    profile template [fnsearch "template.ini" $env(TCLINCLUDE)]
    getTemplateData

    if {[catch {set rulebookFile $env(OPLCONFIG)}]} {
        logmsg ERR "rulebook.tcl: Invalid environment"
	exit 1
    }
 
    logmsg DEBUG "using rulebook $rulebookFile"
    profile rulebook $rulebookFile
}


proc refreshRulebook {} {
    rulebook refresh
    global allSections
    global agentSections
    global mappingRules

    set allSections ""
    set agentSections ""
    set mappingRules ""
}


proc getTemplateData {} {
    global allDatabases
    template readall allDatabases Templates
}


proc getAllSections {} {

    global allSections

    if {$allSections == ""} {
	set allSections [rulebook sections -lower]
    }
    return $allSections
}


proc getAgentSections {} {

    global agentSections

    if {$agentSections == ""} {

	set allSections [getAllSections]

	set skipSections {
	    "request broker"
	    "protocol tcp"
	    "protocol spx"
	    "protocol decnet"
	    "communications"
	    "security"
	    "mapping rules"
	    "domain aliases"
	    "database aliases"
	    "user aliases"
	    "opsys aliases"
	    "machine aliases"
	    "application aliases"
	    "persistent services"
	}

	set agentSections {}
	foreach sect $allSections {
	    # Find all sections not in skipSections,
	    # that are also not an environment section
	    # and have a Program variable

	    if {[lsearch $skipSections $sect] == -1 &&
		[string match "environment *" $sect] == 0 &&
		[rulebook read $sect "Program" ""] != ""} {
		lappend agentSections $sect
	    }
	}

	# Do not include persistent services
	set list [rulebook keys "persistent services"]
	if {[lsearch -exact $allSections "persistent services"] != -1} {
	    rulebook readall psList "persistent services"
	    foreach ps [array names psList] {
		set psa [string tolower $psList($ps)]
		set i [lsearch $agentSections $psa]
		if {$i >= 0} {
		    set agentSections [lreplace $agentSections $i $i]
		}
	    }
	}
    }

    return $agentSections
}


proc getMappingRules {} {
    global mappingRules
    if {$mappingRules == ""} {
	set rules [rulebook keys "Mapping Rules"]
	foreach rule $rules {
	    set value [rulebook read "Mapping Rules" $rule]

	    # 0 domain
	    # 1 database
	    # 2 uid
	    # 3 opsys
	    # 4 machine
	    # 5 application
	    # 6 mode (ro/rw)
	    # 7 act (accept/reject)
	    # 8 argument (agent/reason)
	    set matchrule [split $rule :]

	    set act [lindex $value 0]
	    set acti [lrange $value 1 end]
	    lappend matchrule $act
	    lappend matchrule $acti
	    lappend mappingRules $matchrule
	}
    }
    return $mappingRules
}
