#
#  udbcdsn.tcl
#
#  $Id: udbcdsn.tcl,v 1.19 2003/03/10 12:53:56 source Exp $
#
#  UDBC routines
#
#  (C)Copyright 1998 OpenLink Software.
#  All Rights Reserved.
#
#  The copyright above and this notice must be preserved in all
#  copies of this source code.  The copyright above does not
#  evidence any actual or intended publication of this source code.
#
#  This is unpublished proprietary trade secret of OpenLink Software.
#  This source code may not be copied, disclosed, distributed, demonstrated
#  or licensed except as authorized by OpenLink Software.
#

set dsnList ""
set dsnNameDescList ""

# undefined
set undef "<UNDEF>"

# List of Installed Openlink drivers (filled in at init)
set InstalledDrivers    {}
# List of installed OpenLink DLLs corresponds to InstalledDrivers
set InstalledDLLs {}
# Valid Openlink drivers to look out for (filled in at init)
set OpenlinkDrivers {}

#
# Initialize dsn routines (internal use)
#
proc DSNInit {} {
    global dsninit env
    if {[info exist dsninit]} {
	return 1
    }
    set dir [master rulebook read "Request Broker" BinaryDirectory "."]
    set ini "$dir/udbc.ini"

    if {[info exist env(UDBCINI)]} {
	set ini $env(UDBCINI)
    }

    if {[catch {
	profile udbcini $ini r
	}]} {
	dsnfail "Cannot find OpenLink UDBC.INI file."
	return 0
    }
    set dsninit 1
    return 1
}

#
# Return all UDBC Data Sources ie. List of DSNs
#
proc GetDSNList {} {
    global dsnList
    if {$dsnList == ""} {
	DSNInit
	set allSections [udbcini sections]
	set skipSections {
	    "odbc"
	    "udbc"
	    "communications"
	    "protocol tcp"
	    "protocol spx"
	    "protocol decnet"
	    "server types"
	}

	foreach sect $allSections {
	    # Find all sections not in skipSections
	    if {[lsearch $skipSections [string tolower $sect]] == -1} {
		lappend dsnList $sect
	    }
	}
    }
    return [lsort2 $dsnList]
}

#
# Return all UDBC Data Sources as pairs of {name description}
#
proc GetDSNNameDescList {} {
    global dsnNameDescList
    if {$dsnNameDescList == ""} {
	foreach dsn [GetDSNList] {
	    lappend dsnNameDescList [list $dsn [GetDSNValue $dsn "Description"]]
	}
    }

    return $dsnNameDescList
}

#
# Return the Description as no driver name for a datasource
#
proc GetDSNDriverName {dsn} {
    return [GetDSNValue "Description" $dsn]
}

#
# Get a value from a DSN
#
proc GetDSNValue {dsn var} {
    global undef env

    DSNInit
    set value [udbcini read $dsn $var $undef]
    return $value
}

# Return TRUE if Openlink driver (which can be edited)
proc DSNisOpenlink {dsn} {
    global InstalledDrivers
    return [expr {[lsearch $InstalledDrivers [GetDSNDriverName $dsn]] != -1}]
}

#
# Check if UDBC is installed correctly
#
proc CheckUDBCInstalled {} {
    return [DSNInit]
}

proc dsnfail {reason} {
    html <html><body>
    html <center><h2>Error</h2>
    html <h3>$reason</h3>
    html {<form><input type="button" name="exit" value="Exit" onclick="self.close()"></form></center></body></html>}
}

#
# Return ODBC or UDBC selected by current TCL-DB support
#
proc GetDBC {} {
    if {[database isudbc]} {
	return "UDBC"
    } else {
	return "ODBC"
    }
}

#
# Return list of server types
#
proc GetSVTypeList {} {
    global svTypes svTypesFixed initemplateini env
    set svTypesFixed 0
    set svTypes ""

    catch {
      if {[os type] != "win32"} {
	# get list from templates.ini
        if {![info exist initemplateini]} {
	    profile templateini [fnsearch "template.ini" $env(TCLINCLUDE)]
	    set initemplateini 1
	}
	set svTypes [templateini keys "Server Types"]
      } else {
	# get list from registry
	set svTypes [registry values "HKEY_LOCAL_MACHINE\\Software\\ODBC\\OPENLINK.INI\\Server Types"]
      }
    }
    return [lsort2 $svTypes]
}

#
# Return list of server types as pairs of {type description}
#
proc GetSVTypeDescList {} {
    set svTypeDescList ""

    foreach type [GetSVTypeList] {
	set desc ""
	catch {set desc [GetSVType $type] }
	lappend svTypeDescList [list $type $desc]
    }

    return $svTypeDescList
}

#
# Return Description for a Server type
#
proc GetSVType {st} {
    global initemplateini env
    set desc ""

    catch {
      if {[os type] != "win32"} {
	# get desc from templates.ini
        if {![info exist initemplateini]} {
	    profile templateini [fnsearch "template.ini" $env(TCLINCLUDE)]
	    set initemplateini 1
	}
	set desc [templateini read "Server Types" $st]
      } else {
	# get desc from registry
	set desc [registry get "HKEY_LOCAL_MACHINE\\Software\\ODBC\\OPENLINK.INI\\Server Types" $st]
      }
    }
    return $desc
}

#
# Write a server type and desc (or delete it)
#
proc SetSvType {st desc} {
    global initemplateini undef env
    if {[os type] != "win32"} {
	# write to templates.ini
        if {![info exist initemplateini]} {
	    profile templateini [fnsearch "template.ini" $env(TCLINCLUDE)]
	    set initemplateini 1
	}
	if {$desc == $undef} {
	    templateini write "Server Types" $st
	} else {
	    templateini write "Server Types" $st $desc
	}
	templateini commit
    } else {
	# write to registry
	if {$desc == $undef} {
	    catch { registry delete "HKEY_LOCAL_MACHINE\\Software\\ODBC\\OPENLINK.INI\\Server Types" $st }
	} else {
	    catch { registry set "HKEY_LOCAL_MACHINE\\Software\\ODBC\\OPENLINK.INI\\Server Types" $st $desc }
	}
    }
}

#
# Remove the DSN
#
proc DSNDelete {dsn} {
    udbcini write $dsn
    udbcini commit
}

#
# Save the dsnCfg array.
#
proc DSNSave {dsn} {
    global dsnCfg undef
    foreach i [array names dsnCfg] {
	if {$dsnCfg($i) == $undef} {
	    udbcini write $dsn $i
	} else {
	    udbcini write $dsn $i $dsnCfg($i)
	}
    }
    udbcini commit
}


#
# Return List of drivers for this OS
# Not supported on UDBC
#
proc DSNDriverList {} {
    return {}
}

#
# Return Wizard style for a driver
#
proc DSNDriverWizard {drivername} {
global undef env
    set listsection "[os type] drivers"
    if {[catch {
	profile driversini [fnsearch "drivers.ini" $env(TCLINCLUDE)] r
	}]} {
	dsnfail "Cant read drivers.ini"
	return ""
    }
    set Wizard $undef
    catch { set Wizard [driversini read $listsection $drivername] }
    return $Wizard
}

#
# Return Default value for parameter in Wizard set
#
proc GetWizardDefault {wizard param} {
global undef env
    set value $undef
    set defsection "$wizard Defaults"
    if {[catch {
	profile driversini [fnsearch "drivers.ini" $env(TCLINCLUDE)] r
	}]} {
	dsnfail "Cant read drivers.ini"
	return ""
    }
    catch { set value [driversini read $defsection $param] }
    return $value
}

#
# Return list of parameters in Wizard set
#
proc GetWizardParams {wizard} {
global undef env
    set value {}
    if {[catch {
	profile driversini [fnsearch "drivers.ini" $env(TCLINCLUDE)] r
	set value [driversini keys "$wizard Defaults"]
	}]} {
	dsnfail "Cant read drivers.ini"
    }
    return $value
}

#
# Return DLL(driver) name for a specified driver
# Not supported on UDBC
#
proc GetDSNDriver {drivername} {
    global undef
    return $undef
}


#
#  Once we have iodbcinst functionality we can use the real check
#  but for now we need to use
#
proc CheckValidDSN {dsn} {
  if {[regexp {^.*([{}(),;?\*=!@\\]|\[|\])} $dsn]} {
     return 0;
  }

  return 1;
}
