#
#  setup.tcl
#
#  $Id: setup.tcl,v 1.21 2005/02/08 17:16:23 source Exp $
#
#  Script to prepare the admin assistant for the first run
#
#  (C)Copyright 1997 OpenLink Software.
#  All Rights Reserved.
#
#  The copyright above and this notice must be preserved in all
#  copies of this source code.  The copyright above does not
#  evidence any actual or intended publication of this source code.
#
#  This is unpublished proprietary trade secret of OpenLink Software.
#  This source code may not be copied, disclosed, distributed, demonstrated
#  or licensed except as authorized by OpenLink Software.
#

set SilentMode [expr [lsearch $argv "silent"] >= 0]

if {[os type] == "win32" || [os type] == "os2"} {
    set _exe .exe
} else {
    set _exe ""
}

set cfg(oplrqb_ini)	oplrqb.ini
set cfg(oplrqb)		oplrqb$_exe
set cfg(logfile)	www_sv.log
set cfg(logfull)	n
set cfg(www_sv)		www_sv$_exe
set cfg(www_ini)	www_sv.ini
set cfg(www_root)	{html}
set cfg(www_port)	{8000}
set cfg(www_timeout)	{60}
set cfg(www_include)	{include}
set cfg(www_scripts)	{/scripts}
set cfg(www_cgis)	{/cgi-bin}
set cfg(www_index)	{/index.html}
set cfg(uid)		"admin"
set cfg(uid)		"admin"
set cfg(pwd)		"admin"
set cfg(common)		"include/common.tcl"

set keyshort {ini exe log full svexe svini root port timeout inc scripts cgis index uid pwd}
set keylong {oplrqb_ini oplrqb logfile logfull www_sv www_ini www_root www_port www_timeout www_include www_scripts www_cgis www_index uid pwd}

foreach arg $argv {
    if {[regexp -nocase {(.+)=(.+)} $arg varall varname varvalue]} {
	set keyword [lsearch $keyshort $varname]
	if { $keyword >= 0 } {
	    set cfg([lindex $keylong $keyword]) $varvalue
	}
    }
}

proc abort_setup {} {
    puts "Setup aborted."
    exit 1
}

proc getYN {ask} {
    while 1 {
	puts -nonewline "\n$ask? (y/n/q) "
	set reply [string toupper [gets stdin]]
	if {$reply == "Y"} { return 1 }
	if {$reply == "N"} { return 0 }
	if {$reply == "Q"} abort_setup
    }
}

proc getVar {ask default validate} {
    global SilentMode

    if {$SilentMode} {
	return $default
    }

    while 1 {
	puts -nonewline "\n$ask? \[ENTER=$default] : "
	set reply [string trim [gets stdin]]
	if {$reply == ""} {
	    set reply $default
	}
	if {![catch {uplevel "$validate $reply"} msg]} {
	    return $reply
	}
	puts $msg
    }
}

proc validFile {var} {
    if {![file isfile $var]} {
	error "File $var does not exist"
    }
}

proc validDir {var} {
    if {![file isdirectory $var]} {
	error "$var is not a valid directory"
    }
}

proc validBrkDir {var} {
    global cfg
    validDir $var
    if {![file isfile $var/$cfg(oplrqb)]} {
	error "The file $cfg(oplrqb) not found in this directory"
    }
}

proc validCfgDir {var} {
    global cfg
    validDir $var
    if {![file isfile $var/$cfg(common)]} {
	error "The file $cfg(common) not found in this directory"
    }
}

proc validPort {var} {
    if {$var < 1 || $var > 65535} {
	error "should be between 1 and 65535"
    }
}

proc valid {var} { }

proc yesNo {var} {
    set var [string toupper $var]
    if {$var != "Y" && $var != "N"} {
	error "please answer Y or N"
    }
}

# get the Broker & admin assistant directories
proc getDirectories {} {
    global env cfg _exe

    # Locate the oplrqb executable
    if {[file isfile ../$cfg(oplrqb)]} {
	set oplrqb ../$cfg(oplrqb)
    } elseif {[file isfile bin/$cfg(oplrqb)]} {
	set oplrqb bin/$cfg(oplrqb)
    } else {
	if {[catch {set path .:$env(PATH)}] && [catch {set path .:$env(Path)}]} {
	    set path .
	}
	set oplrqb [fnsearch $cfg(oplrqb) $path ""]
    }

    set bindir [file dirname $oplrqb]

    if {[catch {validBrkDir $bindir}]} {
      set bindir [getVar "What is the OpenLink Broker directory" $bindir validBrkDir]
    }

    cd $bindir
    set cfg(bindir) [pwd]
    set cfgdir $cfg(bindir)/w3config

    if {[catch {validCfgDir $cfgdir}]} {
	set cfgdir bin/w3config
	set cfgdir [getVar "What is the OpenLink Admin Assistant directory" $cfgdir validCfgDir]
	cd $cfgdir
	set cfgdir [pwd]
    }

    cd $cfgdir
    set cfg(cfgdir) [pwd]

    # Find www_sv
    if {[file isfile $cfg(bindir)/www_sv$_exe]} {
      set www_sv www_sv$_exe
    } else {
      set www_sv w3config/www_sv$_exe
    }
    set cfg(www_sv) $www_sv
}

# Open the .ini files
proc openIniFiles {} {
    global cfg
    global SilentMode

    set cfg(www_ini) $cfg(cfgdir)/$cfg(www_ini)
    set cfg(oplrqb_ini) $cfg(bindir)/$cfg(oplrqb_ini)

    if {![file isfile $cfg(oplrqb_ini)]} {
	if {!$SilentMode} {
	    puts ""
	    puts "The setup program could not find a rulebook."
	    if {![getYN "Do you want to create a new (default) rulebook"]} {
		abort_setup
	    }
	}
	set newfile 1
    } else {
	set newfile 0
    }

    if {[catch {profile rulebook $cfg(oplrqb_ini) w}]} {
	puts ""
	puts "The setup program could not open the rulebook."
	puts "Please check your permissions."
	abort_setup
    }

    if {[catch {profile www_ini $cfg(www_ini) w}]} {
	puts ""
	puts "The setup program could not open the server configuration file."
	puts "Please check your permissions."
	abort_setup
    }

    if {$newfile} {
	# Create new default rulebook
	rulebook write "Request Broker" CommandLine ""
	rulebook write "Request Broker" Protocols tcp
	rulebook write "Request Broker" MaxAgents 0
	rulebook write "Request Broker" MaxConnections 0

	rulebook write "Protocol TCP" PingWatchdog Yes
	rulebook write "Protocol TCP" PingInterval 120

	rulebook write "Communications" SendSize 16000
	rulebook write "Communications" ReceiveSize 4096
	rulebook write "Communications" BrokerTimeout 30
	rulebook write "Communications" ReceiveTimeout 120
	rulebook write "Communications" RetryTimeout 2

	rulebook write "Security" StartupBy ".*"
	rulebook write "Security" ShutdownBy ".*"
	rulebook write "Security" ShutdownFrom "localhost"
	rulebook write "Security" ValidUidRange "0, 5000"
    }
    # Leave existing win32 lowercase settings
    if {$newfile || [os type] == "unix" || [os type] == "macosx"} {
	rulebook write "Request Broker" BinaryDirectory $cfg(bindir)
    }
}

# Remove IncludeRulebook settings from rulebook
proc fixOldFormat {} {
    global cfg

    set inclList {}
    while 1 {
	set inclName [rulebook read Security IncludeRulebook ""]
	if {$inclName == ""} break
	set inclList [linsert $inclList -1 $inclName]
	rulebook write Security IncludeRulebook
    }
    if {[llength $inclList] > 0} {
	puts ""
	puts "The Admin Assistant doesn't support included rulebooks."
	puts "If you want to, this setup program can expand them for you, so that"
	puts "you can still use the Admin Assistant."
	if {![getYN "Do you want to expand the 'IncludeRulebook' entries"]} {
	    puts ""
	    puts "Please make a backup of the rulebook and re-run this setup program."
	    abort_setup
	}
    }
    foreach inclName $inclList {
	set tmpName $cfg(bindir)/$inclName
	if {[catch {profile incl $tmpName}]} {
	    if {![getYN "Unable to open $tmpName - skip this entry"]} {
		puts ""
		puts "Please manually correct the rulebook and re-run this setup program."
		abort_setup
	    }
	    continue
	}
	foreach sect [incl sections] {
	    foreach key [incl keys $sect] {
		set value [incl read $sect $key ""]
		rulebook write $sect $key $value
	    }
	}
	incl close
    }
}

# Remove the OpenLink Admin Assistant from the Rulebook
proc removeConfigurator {} {
    global _exe

    # Find Configurator
    foreach item [rulebook keys "Persistent Services"] {
	set prog [rulebook read "Persistent Services" $item]
	set prog [rulebook read $prog Program -]
	# Remove the old entry
	if {$prog == "w3config/www_sv$_exe" || $prog == "www_sv$_exe"} {
	    rulebook write "Persistent Services" $item
	    rulebook write $prog
	}
    }

    # Remove the environment
    # rulebook write "Environment WWW_SV"
}

# Install the OpenLink Configurator in the Rulebook
proc installConfigurator {} {
    global cfg

    # Get commandline settings
    set cfg(www_port) [getVar "TCP/IP Port to use" $cfg(www_port) validPort]
    set cfg(logfile) [getVar "Log File" $cfg(logfile) valid]
    set cfg(logfull) [getVar "Log all requests (y/n)" $cfg(logfull) yesNo]

    set cmdLine ""
    if {$cfg(logfile) != "www_sv.log"} {
	set cmdLine "$cmdLine +logfile $cfg(logfile)"
    }
    if {$cfg(logfull) != "n"} { set cmdLine "$cmdLine +loglevel 6" }

    # Install Configurator section
    rulebook write "Persistent Services" Configurator www_sv
    rulebook write www_sv Program $cfg(www_sv)
    rulebook write www_sv Directory w3config
    rulebook write www_sv CommandLine $cmdLine

    # Setup environment
    # rulebook write www_sv "Environment WWW_SV"

    # Create an empty environment section
    rulebook write "Environment WWW_SV" DUMMY ENTRY
    rulebook write "Environment WWW_SV" DUMMY

    # These variables should already exist in the environment
    #
    # rulebook write "Environment WWW_SV" UDBCINI $cfg(bindir)/udbc.ini
    # if {[os type] != "win32"} {
    #  rulebook write "Environment WWW_SV" ODBCINI $cfg(bindir)/odbc.ini
    #  rulebook write "Environment WWW_SV" ODBCINSTINI $cfg(bindir)/odbcinst.ini
    # }
}

# Modify www_sv.ini
proc updateServerIni {} {
    global cfg

    set users [www_ini keys Users]

    # Autoconvert old-style security
    foreach user $users {
	if {[www_ini read "User $user" Admin 0] == 1} {
	    www_ini write "User $user" Level 3
	    www_ini write "User $user" Admin
	} elseif {[www_ini read "User $user" Level "(undef)"] == "(undef)"} {
	    www_ini write "User $user" Level 2
	}
    }
    # Create admin account
    if {$users == ""} {
	set admin [getVar "Administrator account" $cfg(uid) valid]
	set passw [getVar "Administrator's password" $cfg(pwd) valid]
	www_ini write Users $admin 1
	www_ini write "User $admin" FullName Administrator
	www_ini write "User $admin" Level 3
	www_ini write "User $admin" Password [cryptpw $passw]
    }
    www_ini write Startup HttpPort $cfg(www_port)
    www_ini write Startup HttpRoot $cfg(www_root)
    www_ini write Startup Timeout $cfg(www_timeout)
    www_ini write Startup TclInclude $cfg(www_include)
    www_ini write Startup ScriptDirectories $cfg(www_scripts)
    www_ini write Startup CGIDirectories $cfg(www_cgis)
    www_ini write Startup IndexPage $cfg(www_index)
}


#
#  MAIN
#
if {!$SilentMode} {
    puts {
Welcome to the OpenLink Admin Assistant Setup.

This program will install the HTTP-based OpenLink Admin Assistant, thereby
enabling remote configuration for all OpenLink Server Components (Rule Book,
Service and Database Agents) from any Web Browser.
    }
}

getDirectories
openIniFiles
fixOldFormat
removeConfigurator
installConfigurator
updateServerIni

# Commit changes
rulebook commit
rulebook close
www_ini commit
www_ini close

#
#  Try to find out the real name of this machine
#
set host "<hostname of this machine>"
if {[os type] == "unix" || [os type] == "macosx"} {
  catch {
    set host [exec hostname]
  }
}

if {!$SilentMode} {
    puts ""
    puts ""
    puts "The OpenLink Admin Assistant is now ready for use."
    puts "Please (re)start the OpenLink Request Broker (using the command 'oplrqb -v')"
    puts "and then point your Web Browser to the URL below:"
    puts ""
    puts "    http://$host:$cfg(www_port)"
    puts ""

    if {[os type] != "unix"} {
	puts "Press Enter to continue"
	set d [gets stdin]
    }
}
